from modeltranslation.translator import TranslationOptions, translator

from .models import Blog, Category


class BlogTranslationOptions(TranslationOptions):
    fields = ("title", "body")


class CategoryTranslationOptions(TranslationOptions):
    fields = ("name",)


translator.register(Blog, BlogTranslationOptions)
translator.register(Category, CategoryTranslationOptions)



from django.test import TestCase
from django.utils.translation import override

from migrate_test.app.models import Blog


class PreMigrateTest(TestCase):
    def test_verify_installed_apps(self):
        from django.conf import settings

        self.assertNotIn("modeltrans", settings.INSTALLED_APPS)
        self.assertIn("modeltranslation", settings.INSTALLED_APPS)

    def test_model_fields(self):
        self.assertEquals(
            {field.name for field in Blog._meta.get_fields()},
            {
                "id",
                "title",
                "title_de",
                "title_en",
                "title_fr",
                "title_nl",
                "body",
                "body_de",
                "body_en",
                "body_fr",
                "body_nl",
                "category",
            },
        )

    def test_data_available(self):
        def get_titles():
            return {m.title for m in Blog.objects.all()}

        self.assertEquals(get_titles(), {"Falcon", "Dolphin", "Vulture"})

        with override("de"):
            self.assertEquals(get_titles(), {"Falk", "Delfin", ""})



from django.utils.translation import get_language as _get_language

from .conf import get_available_languages, get_default_language


def get_language():
    """
    Return an active language code that is guaranteed to be in settings.LANGUAGES

    (Django does not seem to guarantee this for us.)
    """
    lang = _get_language()
    if lang in get_available_languages():
        return lang
    return get_default_language()


def split_translated_fieldname(field_name):
    _pos = field_name.rfind("_")
    return (field_name[0:_pos], field_name[_pos + 1 :])


def build_localized_fieldname(field_name, lang):
    if lang == "id":
        # The 2-letter Indonesian language code is problematic with the
        # current naming scheme as Django foreign keys also add "id" suffix.
        lang = "ind"
    return str("{}_{}".format(field_name, lang.replace("-", "_")))



"""
Django settings for migrate_test project.

Generated by 'django-admin startproject' using Django 1.11.1.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import os
import sys

from django.utils.translation import ugettext_lazy as _

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "_84!q(@lf&lovmoj55#(!pw^r!(3ss*-lj7+w5#)f%3j4geg-i"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "modeltranslation",
    # "modeltrans",
    "migrate_test.app",
]

ROOT_URLCONF = "migrate_test.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "migrate_test.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "NAME": "modeltrans-migration",
        "USER": "postgres",
        "PASSWORD": "test1234",
        "HOST": "localhost",
        "PORT": 5432,
        "CONN_MAX_AGE": 600,
        "TEST": {"NAME": "modeltrans-migration"},
    }
}
if "TRAVIS" in os.environ:
    DATABASES["default"].update(
        {
            "NAME": "modeltrans-migration",
            "TEST": {"NAME": "modeltrans-migration"},
            "USER": "postgres",
            "PASSWORD": "",
            "HOST": "localhost",
            "PORT": "",
        }
    )

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = "en-us"

LANGUAGES = [("de", _("German")), ("en", _("English")), ("nl", _("Dutch")), ("fr", _("French"))]

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = "/static/"

if "test" in sys.argv:
    print("\n\033[33mWarning\033[00m: Running without migrations")

    class DisableMigrations(object):
        def __contains__(self, item):
            return True

        def __getitem__(self, item):
            return None

    MIGRATION_MODULES = DisableMigrations()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.apps
from django.apps import AppConfig

from .conf import check_fallback_chain
from .translator import translate_model


class RegistrationConfig(AppConfig):
    name = "modeltrans"
    verbose_name = "Django modeltrans using a registry."

    def ready(self):
        check_fallback_chain()

        for Model in django.apps.apps.get_models():
            translate_model(Model)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import Blog, Category


@admin.register(Blog)
class BlogAdmin(admin.ModelAdmin):
    list_display = ("title", "category")


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# from django.contrib.postgres.indexes import GinIndex
from django.db import models

# from modeltrans.fields import TranslationField


class Category(models.Model):
    name = models.CharField(max_length=255)

    # i18n = TranslationField(fields=("name",), virtual_fields=False)

    class Meta:
        # indexes = [GinIndex(fields=["i18n"])]
        verbose_name_plural = "categories"

    def __str__(self):
        return self.name


class Blog(models.Model):
    title = models.CharField(max_length=255)
    body = models.TextField(null=True, blank=True)
    category = models.ForeignKey(Category, null=True, blank=True, on_delete=models.CASCADE)

    # i18n = TranslationField(fields=("title", "body"), virtual_fields=False)

    class Meta:
        # indexes = [GinIndex(fields=["i18n"])]
        verbose_name_plural = "blogs"

    def __str__(self):
        return self.title



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-16 15:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Blog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("body", models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("name", models.CharField(max_length=255)),
            ],
            options={"verbose_name_plural": "categories"},
        ),
        migrations.AddField(
            model_name="blog",
            name="category",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="app.Category",
            ),
        ),
    ]
